/*** Copyright: OGIHARA Takeshi, April 2008 ***/

#import "UroPiece.h"
#import <Foundation/Foundation.h>
#import <AppKit/NSColor.h>
#import <stdlib.h>
#import <unistd.h>
#import "UroCtrl.h"

#define SpeedFactor  0.3
#define C_HUE        12

@implementation UroPiece

static NSObject *rndLock = nil;
static int bWidth, bHeight;
static int colorOrder = 0;
static float colorBrite = 1.0;

+ (void)initialize
{
    if (rndLock == nil) {
        srandom((unsigned int)getpid());
        rndLock = [[NSObject alloc] init];
        colorOrder = 0;
        colorBrite = 1.0;
    }
}

+ (NSColor *)pieceColor
{
    NSColor *color = [NSColor
            colorWithCalibratedHue: (colorOrder % C_HUE) / (float)C_HUE
            saturation:1.0 brightness:colorBrite alpha:1.0];
    if (++colorOrder >= C_HUE) {
        colorOrder = 0;
        colorBrite *= 0.6;
    }
    return color;
}

- (int)random:(int)range
{
    int v;
    @synchronized(rndLock) {
        v = random() % range;
    }
    return v;
}

+ (void)setBoardWidth:(int)w height:(int)h {
    bWidth = w, bHeight = h;
}

+ (void)connectNewClient:(id)ports
{
    id pool = [[NSAutoreleasePool alloc] init];
    NSConnection *conn = [NSConnection
        connectionWithReceivePort:[ports objectAtIndex:0]
        sendPort:[ports objectAtIndex:1]];
    if (conn) {
        id ctrl, client;
        ctrl = [conn rootProxy];
        [ctrl setProtocolForProxy:@protocol(UroServices)];
        client = [[self alloc] initWithController: ctrl];
        [ctrl registerAsClient:client];
        [client release];
        [[NSRunLoop currentRunLoop] run];
    }
    [pool release];
    [NSThread exit];
}

- (id)initWithController:(id)obj
{
    if ((self = [super init]) != nil) {
        controller = obj;
        lastDirection = [self random:Directions];
        color = [[self class] pieceColor];
        inAction = NO;
    }
    return self;
}

- (UroPoint)reset
{
    location.x = [self random:bWidth];
    location.y = [self random:bHeight];
    return location;
}

- (void)move:(id)sender
{
    int d;
    BOOL ok;
    UroPoint newloc;

    d = [self random: Directions+4];
    if (d >= Directions) d = lastDirection;
    for (ok = NO; ; d = (d+1) % Directions) {
        newloc = location;
        switch (d) {
        case d_Up:    if (++newloc.y < bHeight) ok = YES;
            break;
        case d_Down:  if (--newloc.y >= 0) ok = YES;
            break;
        case d_Right: if (++newloc.x < bWidth) ok = YES;
            break;
        case d_Left:  if (--newloc.x >= 0) ok = YES;
            break;
        }
        if (ok) break;
    }
    lastDirection = d;
    if ([controller moveFrom:location to:newloc])
        location = newloc;
    if (inAction)
        [self start];
}

- (oneway void)start {
    NSTimeInterval delay = [self random:15] / 10.0 + 0.5;
    inAction = YES;
    [self performSelector:@selector(move:)
        withObject:self afterDelay: delay * SpeedFactor];
}

- (oneway void)stop {
    [[self class] cancelPreviousPerformRequestsWithTarget:self
        selector:@selector(move:) object:self];
    inAction = NO;
}

@synthesize color;

@end
