/*** Copyright: OGIHARA Takeshi, Feb. 2008 ***/

#import "UroView.h"
#import <Cocoa/Cocoa.h>
#import "UroPiece.h"

@implementation UroView

- (id)initWithGrid:(int)grid width:(int)wid height:(int)hgt
{
    NSRect rect = NSMakeRect(0.0, 0.0, wid*grid, hgt*grid);
    if ((self = [super initWithFrame:rect]) != nil) {
        gsize = grid, gwidth = wid, gheight = hgt; 
    }
    return self;
}

- (void)dealloc {
    [cacheimg release];
    [super dealloc];
}

- (void)prepareDrawing
{
    NSRect rect = [self frame];
    if (cacheimg == nil)
        cacheimg = [[NSImage alloc] initWithSize:rect.size];
    [cacheimg lockFocus];
    [[NSColor whiteColor] set];
    [NSBezierPath fillRect: rect];
    [cacheimg unlockFocus];
}

- (void)setBoard:(id **)board { uBoard = board; }

- (void)drawFrom:(UroPoint)lowleft to:(UroPoint)upright
{
    int x, y;
    NSRect area, rect;
    int xmin = lowleft.x;
    int ymin = lowleft.y;
    int xmax = upright.x;
    int ymax = upright.y;
    area = NSMakeRect(xmin*gsize, ymin*gsize,
            (xmax-xmin+1)*gsize, (ymax-ymin+1)*gsize);
    [cacheimg lockFocus];
    [[NSColor whiteColor] set];
    [NSBezierPath fillRect:area];
    rect.size = NSMakeSize(gsize-2.0, gsize-2.0);
    for (y = ymin; y <= ymax; y++) {
        rect.origin.y = y*gsize + 1.0;
        for (x = xmin; x <= xmax; x++) {
            id obj = uBoard[y][x];
            if (obj) {
                rect.origin.x = x*gsize + 1.0;
                [[obj color] set];
                [[NSBezierPath bezierPathWithOvalInRect:rect] fill];
            }
        }
    }
    [cacheimg unlockFocus];
    [self setNeedsDisplayInRect:area];
}

/* overwrite */
- (void)drawRect:(NSRect)aRect
{
    [cacheimg compositeToPoint:(aRect.origin)
                      fromRect:aRect operation:NSCompositeCopy];
}

@end
