/*** Copyright: OGIHARA Takeshi, Feb. 2008 ***/

#import <Foundation/Foundation.h>

@interface Creature : NSObject  // Use Garbage Collection
{
    NSString *name;
    int      hitPoint;
}
- (id)initWithName:(NSString *)str hitPoint:(int)n;
@property int hitPoint;
- (void)sufferDamage:(int)val;
- (NSString *)description;
- (void)observeValueForKeyPath:(NSString *)keyPath
    ofObject:(id)object change:(NSDictionary *)change
    context:(void *)context;
@end

@implementation Creature
- (id)initWithName:(NSString *)str hitPoint:(int)n {
    if ((self = [super init]) != nil)
        name = str, hitPoint = n;
    return self;
}

@synthesize hitPoint;

- (void)sufferDamage:(int)val {
    hitPoint = (hitPoint > val) ? (hitPoint - val) : 0;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"<<%@, %@, HP=%d>>",
               NSStringFromClass([self class]), name, hitPoint];
}

- (void)observeValueForKeyPath:(NSString *)keyPath
    ofObject:(id)object change:(NSDictionary *)change
    context:(void *)context
{
    NSLog(@"--- Received by %@ ---\n", self);
    NSLog(@"Object=%@, Path=%@\n", object, keyPath);
    NSLog(@"Change=%@\n", change);
}
@end

@interface Person : Creature
@end

@implementation Person
@end

@interface Dragon : Creature {
    Person   *master;
}
@end

@implementation Dragon
@end

int main(void)
{
    int opt = NSKeyValueObservingOptionOld | NSKeyValueObservingOptionNew;
    Person *p1 = [[Person alloc] initWithName:@"Taro" hitPoint:500];
    Person *p2 = [[Person alloc] initWithName:@"Jiro" hitPoint:140];
    Dragon *dra = [[Dragon alloc] initWithName:@"Choco" hitPoint:400];
    [dra setValue:p1 forKey:@"master"];
    [dra addObserver:dra forKeyPath:@"master.hitPoint"
            options:opt context:NULL];                          // A
    [p1 addObserver:p2 forKeyPath:@"hitPoint"
            options:opt context:NULL];                          // B
    
    [p1 setHitPoint: 800];                                      // 1
    [p1 sufferDamage: 200];                                     // 2
    [dra setValue:p2 forKey:@"master"];                         // 3
    p1.hitPoint -= 100;                                         // 4
    p2.hitPoint += 200;                                         // 5
    [dra removeObserver:dra forKeyPath:@"master.hitPoint"];     // 6
    p2.hitPoint -= 300;                                         // 7
    return 0;
}
